@@ -18,9 +18,10 @@
 
 import os
 import stat
+from typing import List
 
 
-__version__ = (0, 0, 8)
+__version__ = (0, 0, 10)
 
 
 USER_AGENT = "Ognibuild"
@@ -32,6 +33,12 @@ class DetailedFailure(Exception):
         self.argv = argv
         self.error = error
 
+    def __eq__(self, other):
+        return (isinstance(other, type(self)) and
+                self.retcode == other.retcode and
+                self.argv == other.argv and
+                self.error == other.error)
+
 
 class UnidentifiedError(Exception):
     """An unidentified error."""
@@ -42,6 +49,13 @@ class UnidentifiedError(Exception):
         self.lines = lines
         self.secondary = secondary
 
+    def __eq__(self, other):
+        return (isinstance(other, type(self)) and
+                self.retcode == other.retcode and
+                self.argv == other.argv and
+                self.lines == other.lines and
+                self.secondary == other.secondary)
+
     def __repr__(self):
         return "<%s(%r, %r, ..., secondary=%r)>" % (
             type(self).__name__,
@@ -76,6 +90,25 @@ class Requirement(object):
         raise NotImplementedError(self)
 
 
+class OneOfRequirement(Requirement):
+
+    elements: List[Requirement]
+
+    family = 'or'
+
+    def __init__(self, elements):
+        self.elements = elements
+
+    def met(self, session):
+        for req in self.elements:
+            if req.met(session):
+                return True
+        return False
+
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.elements)
+
+
 class UpstreamOutput(object):
     def __init__(self, family):
         self.family = family
