@@ -208,6 +208,18 @@ class NodePackageRequirement(Requirement):
         return "%s(%r)" % (type(self).__name__, self.package)
 
 
+class LuaModuleRequirement(Requirement):
+
+    module: str
+
+    def __init__(self, module):
+        super(LuaModuleRequirement, self).__init__("lua-module")
+        self.module = module
+
+    def __repr__(self):
+        return "%s(%r)" % (type(self).__name__, self.module)
+
+
 class PerlPreDeclaredRequirement(Requirement):
 
     name: str
@@ -360,6 +372,10 @@ class GoPackageRequirement(Requirement):
         self.package = package
         self.version = version
 
+    def __repr__(self):
+        return "%s(%r, version=%r)" % (
+            type(self).__name__, self.package, self.version)
+
     def __str__(self):
         if self.version:
             return "go package: %s (= %s)" % (self.package, self.version)
@@ -716,3 +732,12 @@ class BoostComponentRequirement(Requirement):
     def __init__(self, name):
         super(BoostComponentRequirement, self).__init__("boost-component")
         self.name = name
+
+
+class GnulibDirectoryRequirement(Requirement):
+
+    directory: str
+
+    def __init__(self, directory):
+        super(GnulibDirectoryRequirement, self).__init__("gnulib")
+        self.directory = directory
