@@ -23,9 +23,9 @@ import unittest
 
 def test_suite():
     names = [
-        "debian_build",
     ]
     if os.path.exists("/usr/bin/dpkg-architecture"):
+        names.append("debian_build")
         names.append("debian_fix_build")
     module_names = ["ognibuild.tests.test_" + name for name in names]
     loader = unittest.TestLoader()
